package LATMOS::Accounts::Web::Controller::Users;

use strict;
use warnings;
use base 'Catalyst::Controller';

=head1 NAME

LATMOS::Accounts::Web::Controller::Users - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index 

=cut

sub index : Private {
    my ( $self, $c ) = @_;

    $c->stash->{ofilter} = $c->model('AttrFilter', 'user');
    $c->stash->{page}{title} = "Liste des utilisateurs";
}


sub change_password : Private {
    my ($self, $c, $username, $password) = @_;

    my $base = $c->model('Accounts')->db;
    $c->stash->{user} = $base->get_object('user', $username) or do {
        $c->forward('/no_object');
        return "Cannot get user object";
    };

    my $sync = $c->model('Accounts')->sync_access;

    # This is widelly burk:
    # synchaccess do not manage connect() at time, opening a
    # backdoor then :\
    foreach ($sync->bases) { $_->{_user} = $username }

    my $suser = $sync->get_object('user', $username);
    my $msg = $c->stash->{user}->check_password($password);
    if ($msg ne 'ok') {
        return $msg . ', mot de passe inchangé';
    } elsif ($suser && $suser->_set_password($password)) {
        $sync->commit;
        return;
    } else {
        return 'Erreur lors du changement de mot de passe';
    }
}

sub default : LocalPath {
    my ( $self, $c, undef, $username, $subform, $arg ) = @_;

    my $base = $c->model('Accounts')->db;

    $c->stash->{page}{title} = "Utilisateur :: $username";
    $c->stash->{username} = $username;
    $c->stash->{user} = $base->get_object('user', $username) or do {
        $c->forward('/no_object');
        return;
    };
    $c->stash->{subform} = $subform || '';
    $c->stash->{page}{title} = "Utilisateur :: $username";

    if ($c->req->param('make_active')) {
        $c->stash->{user}->set_c_fields('exported' => 1);
        $base->commit;
    }
    if ($c->req->param('make_inactive')) {
        $c->stash->{user}->set_c_fields('exported' => 0);
        $base->commit;
    }
    if ($c->req->param('delete')) {
        $base->delete_object('user', $username);
        $base->commit;
        $c->res->redirect('/users');
    }

    for ($subform || '') {
        # Exceptions...
        /^passwd$/ and do {
            $c->stash->{template} = 'users/passwd.tt';
            if ($c->req->param('passwd')) {
                if ($base->check_acl($c->stash->{user}, 'userPassword', 'w')) {
                    if ($c->req->param('passwd') eq ($c->req->param('cpasswd') || '')) {
                        $c->stash->{pmerror} = $c->forward('change_password',
                            [ $username, $c->req->param('passwd') ] ) || 'Mot de passe changé';
                    } else {
                        $c->stash->{pmerror} = 'Mot de passe différents';
                    }
                }
            }
            last;
        };
        /^groups$/ and do {
            $c->stash->{template} = 'users/groups.tt';
            # list of group for which users can be added
            my %ingroups = map { $_ => 1 } @{ $c->stash->{user}->get_c_field('memberOf') || []};
            if ($c->req->param('addgroup')) {
                $c->stash->{user}->set_c_fields('memberOf', [ keys %ingroups, $c->req->param('addgroup') ]);
                $c->stash->{user}->base->commit;
                # reread:
                %ingroups = map { $_ => 1 } @{ $c->stash->{user}->get_c_field('memberOf') || []};
            } elsif ($c->req->param('delgroup')) {
                $c->stash->{user}->set_c_fields('memberOf',
                    [ grep { $_ ne $c->req->param('delgroup') } keys %ingroups ]
                );
                $c->stash->{user}->base->commit;
                %ingroups = map { $_ => 1 } @{ $c->stash->{user}->get_c_field('memberOf') || []};
            }

            $c->stash->{othergroups} = [ grep { ! $ingroups{$_} } $base->list_objects('group') ];
            last;
        };
        /^address$/ and do {
            if ($arg) {
                $c->stash->{address} = $base->get_object('address', $arg);
            } else {

                if ($c->req->param('delete')) {
                    $base->delete_object('address', $c->req->param('delete'));
                    $base->commit;
                } elsif ($c->req->param('add_addr')) {
                    $c->stash->{form} = $c->model('AttrForms', 'address', 'address',
                        $base);
                    my $addname = $username . join('', map
                        {('a'..'z')[rand(26)]}(0..4));
                    if ($base->create_c_object('address',
                        $addname,
                        (map { $_ => ($c->req->param($_) || undef) }
                        $c->stash->{form}->attributes('w')),
                        user => $username,
                    )) {
                        $base->commit;
                        $c->res->redirect($c->uri_for($username, 'address',
                                $addname));
                        return;
                    }
                } elsif ($c->req->param('main')) {
                    if (my $ad = $base->get_object('address',
                            $c->req->param('main'))) {
                        warn $c->req->param('main');
                        warn $ad->set_c_fields('isMainAddress' => 1);
                        $ad->base->commit;
                    }
                }

                my ($main) = $c->stash->{user}->get_attributes('otheraddress');
                $c->stash->{address} = $base->get_object('address', $main);
            }
            if ($c->stash->{address}) {
                $c->stash->{form} = $c->model('AttrForms', 'address',
                    $c->stash->{address});
                $c->stash->{form}->set_attrs;
            }
            $c->stash->{template} = 'users/address.tt';

            last;
        };
        /^mail$/ and do {

            $c->stash->{db} = $base;
            $c->stash->{template} = 'users/mail.tt';

            if ($c->req->param('usermail')) {
                $c->stash->{user}->set_c_fields(
                    mail => $c->req->param('mail') || undef,
                    aliases => [ grep { $_ } $c->req->param('aliases') ],
                    revaliases => $c->req->param('revaliases') || undef,
                    forward => $c->req->param('forward') || undef,
                ) and $base->commit;
            }

            last;
        };
        /^my$/ and do {
            $c->stash->{db} = $base;
            $c->stash->{template} = 'users/my.tt';
        };
        /^dump$/ and do {
            $c->stash->{db} = $base;
            $c->stash->{template} = 'users/dump.tt'
            #$c->res->body($c->stash->{user}->dump({only_rw => 1}));
        };

        $c->stash->{form} = $c->model('AttrForms', 'user' . $c->stash->{subform}, $c->stash->{user});
        $c->stash->{form}->set_attrs;
    }
}

=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
